@@ -1,13 +1,13 @@
-#include <Rcpp.h>
-
 #include "utilities.h"
 #include "dataframe_list.h"
 
+#include <Rcpp.h>
+
 #include <cstddef>    // size_t
 #include <vector>     // vector
 #include <string>     // string
 #include <algorithm>  // fill, lower_bound, max_element, memmove, 
-// min_element, sort, swap, upper_bound
+                      // min_element, sort, swap, upper_bound
 #include <numeric>    // accumulate, inner_product, iota 
 #include <functional> // function
 #include <cmath>      // copysign, exp, fabs, isinf, isnan, log, pow, sqrt
@@ -28,6 +28,16 @@
 double boost_pnorm(double q, double mean, double sd, bool lower_tail) {
   if (std::isnan(q)) return std::numeric_limits<double>::quiet_NaN();
   if (sd <= 0) throw std::invalid_argument("Standard deviation must be positive.");
+  
+  double z = (q - mean) / sd;
+  if (lower_tail) {
+    if (z <= -EXTREME_Z) return 0.0;
+    if (z >= EXTREME_Z) return 1.0;
+  } else {
+    if (z >= EXTREME_Z) return 0.0;
+    if (z <= -EXTREME_Z) return 1.0;
+  }
+  
   boost::math::normal_distribution<> dist(mean, sd);
   if (lower_tail) return boost::math::cdf(dist, q);
   else return boost::math::cdf(boost::math::complement(dist, q));
@@ -38,8 +48,38 @@ double boost_qnorm(double p, double mean, double sd, bool lower_tail) {
   if (sd <= 0) throw std::invalid_argument("Standard deviation must be positive.");
   if (p < 0.0 || p > 1.0) throw std::invalid_argument(
       "Probability must be between 0 and 1.");
+  
+  // Clamp extreme probabilities to avoid overflow in boost::math::quantile
+  bool at_extreme = false;
+  double extreme_quantile = 0.0;
+  
+  if (lower_tail) {
+    if (p <= MIN_PROB) {
+      at_extreme = true;
+      extreme_quantile = MIN_NORMAL_QUANTILE;
+    } else if (p >= MAX_PROB) {
+      at_extreme = true;
+      extreme_quantile = MAX_NORMAL_QUANTILE;
+    }
+  } else {
+    // When lower_tail = false, we compute quantile(1 - p)
+    if (p <= MIN_PROB) {
+      at_extreme = true;
+      extreme_quantile = MAX_NORMAL_QUANTILE;
+    } else if (p >= MAX_PROB) {
+      at_extreme = true;
+      extreme_quantile = MIN_NORMAL_QUANTILE;
+    }
+  }
+  
+  // If at extreme, return scaled value directly
+  if (at_extreme) {
+    return mean + sd * extreme_quantile;
+  }
+  
+  // Safe to call boost quantile
   boost::math::normal_distribution<> dist(mean, sd);
-  return lower_tail ? boost::math::quantile(dist, p) : 
+  return lower_tail ? boost::math::quantile(dist, p) :
     boost::math::quantile(dist, 1.0 - p);
 }
 
@@ -161,7 +201,6 @@ std::vector<int> which(const std::vector<unsigned char>& vec) {
   return indices;
 }
 
-// findInterval3: adapted from previous implementation
 std::vector<int> findInterval3(const std::vector<double>& x,
                                const std::vector<double>& v,
                                bool rightmost_closed,
@@ -279,7 +318,7 @@ double bisect(const std::function<double(double)>& f,
 // --------------------------- Quantiles -------------------------------------
 
 double quantilecpp(const std::vector<double>& x, double p) {
-  int n = x.size();
+  int n = static_cast<int>(x.size());
   if (n == 0) throw std::invalid_argument("Empty vector");
   if (p < 0.0 || p > 1.0) throw std::invalid_argument("p must be in [0,1]");
   std::vector<double> y(x);
@@ -289,9 +328,7 @@ double quantilecpp(const std::vector<double>& x, double p) {
   double g = h - j;
   if (j <= 0) return y.front();
   if (j >= n) return y.back();
-  double lower_val = y[j - 1];
-  double upper_val = y[j];
-  return (1 - g) * lower_val + g * upper_val;
+  return (1 - g) * y[j - 1] + g * y[j];
 }
 
 double squantilecpp(const std::function<double(double)>& S, double p, double tol) {
@@ -303,7 +340,7 @@ double squantilecpp(const std::function<double(double)>& S, double p, double tol
     if (upper > 1e12) throw std::runtime_error(
         "Cannot find suitable upper bound for quantile search");
   }
-  auto f = [&S, p](double t) -> double { return S(t) - p; };
+  auto f = [&](double t) -> double { return S(t) - p; };
   return brent(f, lower, upper, tol);
 }
 
@@ -348,7 +385,7 @@ ListCpp bygroup(const DataFrameCpp& data,
   std::vector<unsigned char> bool_flat;
   std::vector<std::string> str_flat;
   
-  ListCpp lookups_per_variable; // will contain DataFrameCpp for each variable
+  ListCpp lookups_per_variable; // will contain a std::vector for each variable
   
   for (int i = 0; i < p; ++i) {
     const std::string& var = variables[i];
@@ -367,12 +404,8 @@ ListCpp bygroup(const DataFrameCpp& data,
       var_info[i] = VarLookupInfo{0, off};
       
       // fill indices column i (column-major layout)
-      for (int r = 0; r < n; ++r) {
-        indices(r, i) = idx[r];
-      }
-      
+      intmatrix_set_column(indices, i, idx);
       lookups_per_variable.push_back(std::move(w), var);
-      
     } else if (data.numeric_cols.count(var)) {
       const auto& col = data.numeric_cols.at(var);
       auto w = unique_sorted(col);
@@ -383,12 +416,8 @@ ListCpp bygroup(const DataFrameCpp& data,
       dbl_flat.insert(dbl_flat.end(), w.begin(), w.end());
       var_info[i] = VarLookupInfo{1, off};
       
-      for (int r = 0; r < n; ++r) {
-        indices(r, i) = idx[r];
-      }
-      
+      intmatrix_set_column(indices, i, idx);
       lookups_per_variable.push_back(std::move(w), var);
-      
     } else if (data.bool_cols.count(var)) {
       const auto& col = data.bool_cols.at(var);
       auto w = unique_sorted(col);
@@ -396,15 +425,11 @@ ListCpp bygroup(const DataFrameCpp& data,
       auto idx = matchcpp(col, w);
       
       int off = bool_flat.size();
-      for (bool bv : w) bool_flat.push_back(bv ? 1u : 0u);
+      bool_flat.insert(bool_flat.end(), w.begin(), w.end());
       var_info[i] = VarLookupInfo{2, off};
       
-      for (int r = 0; r < n; ++r) {
-        indices(r, i) = idx[r];
-      }
-      
+      intmatrix_set_column(indices, i, idx);
       lookups_per_variable.push_back(std::move(w), var);
-      
     } else if (data.string_cols.count(var)) {
       const auto& col = data.string_cols.at(var);
       auto w = unique_sorted(col);
@@ -415,12 +440,8 @@ ListCpp bygroup(const DataFrameCpp& data,
       str_flat.insert(str_flat.end(), w.begin(), w.end());
       var_info[i] = VarLookupInfo{3, off};
       
-      for (int r = 0; r < n; ++r) {
-        indices(r, i) = idx[r];
-      }
-      
+      intmatrix_set_column(indices, i, idx);
       lookups_per_variable.push_back(std::move(w), var);
-      
     } else {
       throw std::invalid_argument("Unsupported variable type in bygroup: " + var);
     }
@@ -430,23 +451,23 @@ ListCpp bygroup(const DataFrameCpp& data,
   std::vector<int> combined_index(n, 0);
   int orep = 1;
   for (int i = 0; i < p; ++i) orep *= nlevels[i];
+  int lookup_nrows = orep;
+  
   for (int i = 0; i < p; ++i) {
     orep /= nlevels[i];
+    const int* col_ptr = indices.data_ptr() + i * n;
     for (int j = 0; j < n; ++j) {
-      combined_index[j] += indices(j, i) * orep;
+      combined_index[j] += col_ptr[j] * orep;
     }
   }
   
-  int lookup_nrows = 1;
-  for (int i = 0; i < p; ++i) lookup_nrows *= nlevels[i];
-  
   // Build lookup_df with columns repeated in the same pattern as before.
   DataFrameCpp lookup_df;
+  int repeat_each = lookup_nrows;
   for (int i = 0; i < p; ++i) {
     const std::string& var = variables[i];
     int nlevels_i = nlevels[i];
-    int repeat_each = 1;
-    for (int j = i + 1; j < p; ++j) repeat_each *= nlevels[j];
+    repeat_each /= nlevels_i;
     int times = lookup_nrows / ( nlevels_i * repeat_each );
     
     VarLookupInfo info = var_info[i];
@@ -474,9 +495,7 @@ ListCpp bygroup(const DataFrameCpp& data,
       int idxw = 0;
       for (int t = 0; t < times; ++t) {
         for (int level = 0; level < nlevels_i; ++level) {
-          for (int r = 0; r < repeat_each; ++r) {
-            col[idxw++] = (base[level] != 0);
-          }
+          for (int r = 0; r < repeat_each; ++r) col[idxw++] = base[level];
         }
       }
       lookup_df.push_back(std::move(col), var);
@@ -542,15 +561,67 @@ FlatMatrix mat_mat_mult(const FlatMatrix& A, const FlatMatrix& B) {
   return C;
 }
 
-FlatMatrix transpose(const FlatMatrix& A) {
-  if (A.nrow == 0 || A.ncol == 0) return FlatMatrix();
-  FlatMatrix At(A.ncol, A.nrow);
-  for (int c = 0; c < A.ncol; ++c) {
-    for (int r = 0; r < A.nrow; ++r) {
-      At(c, r) = A(r, c);
+// Transpose a FlatMatrix (double)
+FlatMatrix transpose(const FlatMatrix& M) {
+  if (M.nrow == 0 || M.ncol == 0) return FlatMatrix();
+  
+  const int src_nrow = M.nrow;
+  const int src_ncol = M.ncol;
+  FlatMatrix out(src_ncol, src_nrow); // swapped dims
+  
+  const double* src = M.data_ptr();
+  double* dst = out.data_ptr();
+  
+  for (int c = 0; c < src_ncol; ++c) {
+    const double* src_col = src + c * src_nrow;
+    for (int r = 0; r < src_nrow; ++r) {
+      dst[r * src_ncol + c] = src_col[r];
     }
   }
-  return At;
+  
+  return out;
+}
+
+// Transpose an IntMatrix (int)
+IntMatrix transpose(const IntMatrix& M) {
+  if (M.nrow == 0 || M.ncol == 0) return IntMatrix();
+  
+  const int src_nrow = M.nrow;
+  const int src_ncol = M.ncol;
+  IntMatrix out(src_ncol, src_nrow); // swapped dims
+  
+  const int* src = M.data_ptr();
+  int* dst = out.data_ptr();
+  
+  for (int c = 0; c < src_ncol; ++c) {
+    const int* src_col = src + c * src_nrow;
+    for (int r = 0; r < src_nrow; ++r) {
+      dst[r * src_ncol + c] = src_col[r];
+    }
+  }
+  
+  return out;
+}
+
+// Transpose a BoolMatrix (unsigned char)
+BoolMatrix transpose(const BoolMatrix& M) {
+  if (M.nrow == 0 || M.ncol == 0) return BoolMatrix();
+  
+  const int src_nrow = M.nrow;
+  const int src_ncol = M.ncol;
+  BoolMatrix out(src_ncol, src_nrow); // swapped dims
+  
+  const unsigned char* src = M.data_ptr();
+  unsigned char* dst = out.data_ptr();
+  
+  for (int c = 0; c < src_ncol; ++c) {
+    const unsigned char* src_col = src + c * src_nrow;
+    for (int r = 0; r < src_nrow; ++r) {
+      dst[r * src_ncol + c] = src_col[r];
+    }
+  }
+  
+  return out;
 }
 
 double quadsym(const std::vector<double>& u, const FlatMatrix& v) {
@@ -570,9 +641,11 @@ double quadsym(const std::vector<double>& u, const FlatMatrix& v) {
   }
   return sum;
 }
+
 // --------------------------- Linear algebra helpers (FlatMatrix-backed) ----
 // cholesky2: in-place working on FlatMatrix (n x n), returns rank * nonneg
 int cholesky2(FlatMatrix& matrix, int n, double toler) {
+  double* base = matrix.data_ptr();
   double eps = 0.0;
   for (int i = 0; i < n; ++i) {
     double val = matrix(i, i);
@@ -583,18 +656,20 @@ int cholesky2(FlatMatrix& matrix, int n, double toler) {
   int rank = 0;
   
   for (int i = 0; i < n; ++i) {
-    double pivot = matrix(i, i);
+    double* col_i = base + i * n;
+    double pivot = col_i[i];
     if (std::isinf(pivot) || pivot < eps) {
-      matrix(i, i) = 0.0;
+      col_i[i] = 0.0;
       if (pivot < -8.0 * eps) nonneg = -1;
     } else {
       ++rank;
       for (int j = i + 1; j < n; ++j) {
-        double temp = matrix(i, j) / pivot;
-        matrix(i, j) = temp;
-        matrix(j, j) -= temp * temp * pivot;
+        double* col_j = base + j * n;
+        double temp = col_i[j] / pivot;
+        col_i[j] = temp;
+        col_j[j] -= temp * temp * pivot;
         for (int k = j + 1; k < n; ++k) {
-          matrix(j, k) -= temp * matrix(i, k);
+          col_j[k] -= temp * col_i[k];
         }
       }
     }
@@ -603,75 +678,42 @@ int cholesky2(FlatMatrix& matrix, int n, double toler) {
 }
 
 // chsolve2 assumes matrix holds the representation produced by cholesky2
-void chsolve2(FlatMatrix& matrix, int n, std::vector<double>& y) {
+void chsolve2(FlatMatrix& matrix, int n, double* y) {
   // Forward substitution L * z = y
-  for (int i = 0; i < n; ++i) {
-    double temp = y[i];
-    for (int j = 0; j < i; ++j) temp -= y[j] * matrix(j, i);
-    y[i] = temp;
+  double* base = matrix.data_ptr();
+  for (int j = 0; j < n-1; ++j) {
+    double yj = y[j];
+    if (yj == 0.0) continue;
+    double* col_j = base + j * n;
+    for (int i = j + 1; i < n; ++i) {
+      y[i] -= yj * col_j[i];
+    }
   }
-  // Backward substitution L^T * x = z
+  // Now y holds z; solve L^T * x = z
   if (n == 0) return;
   for (int i = n - 1; i >= 0; --i) {
-    double diag = matrix(i, i);
+    double* col_i = base + i * n;
+    double diag = col_i[i];
     if (diag == 0.0) {
       y[i] = 0.0;
     } else {
       double temp = y[i] / diag;
-      for (int j = i + 1; j < n; ++j) temp -= y[j] * matrix(i, j);
+      for (int j = i + 1; j < n; ++j) temp -= y[j] * col_i[j];
       y[i] = temp;
     }
   }
 }
 
-// chinv2: invert after decomposition in-place (FlatMatrix)
-void chinv2(FlatMatrix& matrix, int n) {
-  // Step 1: invert diagonal and apply sweep operator
-  for (int i = 0; i < n; ++i) {
-    double mii = matrix(i, i);
-    if (mii > 0.0) {
-      matrix(i, i) = 1.0 / mii;
-      for (int j = i + 1; j < n; ++j) {
-        int idx_ij = j * n + i;
-        matrix.data[idx_ij] = -matrix.data[idx_ij];
-        for (int k = 0; k < i; ++k) {
-          matrix(k, j) += matrix.data[idx_ij] * matrix(k, i);
-        }
-      }
-    }
-  }
-  // Step 2: finalize inverse and symmetrize
-  for (int i = 0; i < n; ++i) {
-    double mii = matrix(i, i);
-    if (mii == 0.0) {
-      for (int j = 0; j < i; ++j) matrix(i, j) = 0.0;
-      for (int j = i; j < n; ++j) matrix(j, i) = 0.0;
-    } else {
-      for (int j = i + 1; j < n; ++j) {
-        double temp = matrix(i, j) * matrix(j, j);
-        matrix(j, i) = temp;
-        for (int k = i; k < j; ++k) 
-          matrix(k, i) += temp * matrix(k, j);
-      }
-    }
-  }
-}
-
-// invsympd: invert symmetric positive definite matrix (returns FlatMatrix)
 FlatMatrix invsympd(const FlatMatrix& matrix, int n, double toler) {
   FlatMatrix v = matrix; // copy
-  if (cholesky2(v, n, toler) != 0) {
-    // proceed: cholesky2 returns rank * nonneg; if not PD, chinv2 may not work
-    // still attempt inverse; caller should handle exceptions if needed
+  cholesky2(v, n, toler);
+  FlatMatrix iv(n, n);
+  for (int i = 0; i < n; ++i) {
+    iv(i,i) = 1.0;
+    double* ycol = iv.data_ptr() + i * n;
+    chsolve2(v, n, ycol);
   }
-  chinv2(v, n);
-  // fill symmetric entries (upper -> lower)
-  for (int i = 1; i < n; ++i) {
-    for (int j = 0; j < i; ++j) {
-      v(j, i) = v(i, j);
-    }
-  }
-  return v;
+  return iv;
 }
 
 // -------------------------- Survival helpers --------------------------------
@@ -732,14 +774,54 @@ DataFrameCpp survsplitcpp(const std::vector<double>& tstart,
   return df;
 }
 
+
+//' @title Split a survival data set at specified cut points
+//' @description For a given survival dataset and specified cut times, 
+//' each record is split into multiple subrecords at each cut time. 
+//' The resulting dataset is in counting process format, with each 
+//' subrecord containing a start time, stop time, and event status.
+//' This is adapted from the survsplit.c function from the survival package.
+//'
+//' @param tstart The starting time of the time interval for 
+//'   counting-process data.
+//' @param tstop The stopping time of the time interval for 
+//'   counting-process data.
+//' @param cut The vector of cut points.
+//'
+//' @return A data frame with the following variables:
+//'
+//' * \code{row}: The row number of the observation in the input data 
+//'   (starting from 0).
+//'
+//' * \code{start}: The starting time of the resulting subrecord.
+//'
+//' * \code{end}: The ending time of the resulting subrecord.
+//'
+//' * \code{censor}: Whether the subrecord lies strictly within a record
+//'   in the input data (1 for all but the last interval and 0 for the 
+//'   last interval).
+//'
+//' * \code{interval}: The interval number derived from cut (starting 
+//'   from 0 if the interval lies to the left of the first cutpoint).
+//'
+//' @author Kaifeng Lu, \email{kaifenglu@@gmail.com}
+//'
+//' @keywords internal
+//'
+//' @examples
+//'
+//' survsplit(15, 60, c(10, 30, 40))
+//'
+//' @export
 // [[Rcpp::export]]
-Rcpp::DataFrame survsplitRcpp(const std::vector<double>& tstart,
-                              const std::vector<double>& tstop,
-                              const std::vector<double>& cut) {
-  DataFrameCpp dfcpp = survsplitcpp(tstart, tstop, cut);
+Rcpp::DataFrame survsplit(const std::vector<double>& tstart,
+                          const std::vector<double>& tstop,
+                          const std::vector<double>& cut) {
+  auto dfcpp = survsplitcpp(tstart, tstop, cut);
   return Rcpp::wrap(dfcpp);
 }
 
+
 // ------------------------- QR and other helpers -----------------------------
 double sumsq(const std::vector<double>& x) {
   double s = 0.0;
@@ -1000,9 +1082,7 @@ double qtpwexpcpp1(const double p,
                    const double lowerBound,
                    const bool lowertail,
                    const bool logp) {
-  int m = piecewiseSurvivalTime.size();
-  if (m == 0 || static_cast<int>(lambda.size()) != m) 
-    throw std::invalid_argument("Invalid piecewise model inputs.");
+  int m = static_cast<int>(piecewiseSurvivalTime.size());
   double u = logp ? std::exp(p) : p;
   if (!lowertail) u = 1.0 - u;
   if (u <= 0.0) return lowerBound;
